@@ -248,7 +248,9 @@ module Puppet::Pops::Evaluator::Runtime3Support
     # this is done from the perspective of the environment.
     loader = loaders.private_environment_loader
     if loader && func = loader.load(:function, name)
-      return func.call(scope, *args, &block)
+      Puppet::Util::Profiler.profile(name, [:functions, name]) do
+        return func.call(scope, *args, &block)
+      end
     end
 
     # Call via 3x API if function exists there
@@ -267,9 +269,16 @@ module Puppet::Pops::Evaluator::Runtime3Support
     loaders = scope.compiler.loaders
     # find the loader that loaded the code, or use the private_environment_loader (sees env + all modules)
     adapter = Puppet::Pops::Utils.find_adapter(o, Puppet::Pops::Adapters::LoaderAdapter)
+
+    # Use source location to determine calling module, or use the private_environment_loader (sees env + all modules)
+    # This is necessary since not all .pp files are loaded by a Puppet::Pops::Loader (see PUP-1833)
+    adapter ||= Puppet::Pops::Adapters::LoaderAdapter.adapt_by_source(scope, o)
+
     loader = adapter.nil? ? loaders.private_environment_loader : adapter.loader
     if loader && func = loader.load(:function, name)
-      return func.call(scope, *args, &block)
+      Puppet::Util::Profiler.profile(name, [:functions, name]) do
+        return func.call(scope, *args, &block)
+      end
     end
 
     # Call via 3x API if function exists there
